# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/11/21 17:48
describe: 技术分析相关信号的计算
"""
import numpy as np
from collections import OrderedDict
from datetime import datetime

from czsc import analyze
from ..objects import Signal
from ..enum import Freq
from ..utils.ta import MACD, SMA


def get_s_single_k(c: analyze.CZSC, di: int = 1) -> OrderedDict:
    """获取倒数第i根K线的单K信号"""
    if c.freq not in [Freq.D, Freq.W]:
        return OrderedDict()

    if len(c.bars_raw) < di:
        return OrderedDict()

    s = OrderedDict()
    freq: Freq = c.freq
    k1 = str(freq.value)
    default_signals = [
        Signal(k1=k1, k2=f"倒{di}K", k3="状态", v1="其他", v2='其他', v3='其他'),
    ]
    for signal in default_signals:
        s[signal.key] = signal.value

    k = c.bars_raw[-di]
    if k.close > k.open:
        v = Signal(k1=k1, k2=f"倒{di}K", k3="状态", v1="上涨")
    else:
        v = Signal(k1=k1, k2=f"倒{di}K", k3="状态", v1="下跌")
    s[v.key] = v.value
    return s


def get_s_three_k(c: analyze.CZSC, di: int = 1) -> OrderedDict:
    """倒数第i根K线的三K信号

    :param c: CZSC 对象
    :param di: 最近一根K线为倒数第i根
    :return: 信号字典
    """
    assert di >= 1
    freq: Freq = c.freq
    k1 = str(freq.value)
    k2 = f"倒{di}K"

    s = OrderedDict()
    v = Signal(k1=k1, k2=k2, k3="三K形态", v1="其他", v2='其他', v3='其他')
    s[v.key] = v.value

    if len(c.bars_ubi) < 3 + di:
        return s

    if di == 1:
        tri = c.bars_ubi[-3:]
    else:
        tri = c.bars_ubi[-3 - di + 1:-di + 1]

    if tri[0].high > tri[1].high < tri[2].high:
        v = Signal(k1=k1, k2=k2, k3="三K形态", v1="底分型")
    elif tri[0].high < tri[1].high < tri[2].high:
        v = Signal(k1=k1, k2=k2, k3="三K形态", v1="向上走")
    elif tri[0].high < tri[1].high > tri[2].high:
        v = Signal(k1=k1, k2=k2, k3="三K形态", v1="顶分型")
    elif tri[0].high > tri[1].high > tri[2].high:
        v = Signal(k1=k1, k2=k2, k3="三K形态", v1="向下走")
    else:
        v = None

    if v and "其他" not in v.value:
        s[v.key] = v.value

    return s


def get_s_macd(c: analyze.CZSC, di: int = 1) -> OrderedDict:
    """获取倒数第i根K线的MACD相关信号"""
    freq: Freq = c.freq
    s = OrderedDict()

    k1 = str(freq.value)
    k2 = f"倒{di}K"
    default_signals = [
        Signal(k1=k1, k2=k2, k3="DIF多空", v1="其他", v2='其他', v3='其他'),
        Signal(k1=k1, k2=k2, k3="DIF方向", v1="其他", v2='其他', v3='其他'),

        Signal(k1=k1, k2=k2, k3="DEA多空", v1="其他", v2='其他', v3='其他'),
        Signal(k1=k1, k2=k2, k3="DEA方向", v1="其他", v2='其他', v3='其他'),

        Signal(k1=k1, k2=k2, k3="MACD多空", v1="其他", v2='其他', v3='其他'),
        Signal(k1=k1, k2=k2, k3="MACD方向", v1="其他", v2='其他', v3='其他'),
    ]
    for signal in default_signals:
        s[signal.key] = signal.value

    if len(c.bars_raw) < 100:
        return s

    if di == 1:
        close = np.array([x.close for x in c.bars_raw[-100:]])
    else:
        close = np.array([x.close for x in c.bars_raw[-100-di+1:-di+1]])
    dif, dea, macd = MACD(close, fastperiod=12, slowperiod=26, signalperiod=9)

    # DIF 多空信号
    dif_base = sum([abs(dif[-2] - dif[-1]), abs(dif[-3] - dif[-2]), abs(dif[-4] - dif[-3])]) / 3
    if dif[-1] > dif_base:
        v = Signal(k1=k1, k2=k2, k3="DIF多空", v1="多头")
    elif dif[-1] < -dif_base:
        v = Signal(k1=k1, k2=k2, k3="DIF多空", v1="空头")
    else:
        v = Signal(k1=k1, k2=k2, k3="DIF多空", v1="模糊")
    s[v.key] = v.value

    if dif[-1] > dif[-2] > dif[-3]:
        v = Signal(k1=k1, k2=k2, k3="DIF方向", v1="向上")
    elif dif[-1] < dif[-2] < dif[-3]:
        v = Signal(k1=k1, k2=k2, k3="DIF方向", v1="向下")
    else:
        v = Signal(k1=k1, k2=k2, k3="DIF方向", v1="模糊")
    s[v.key] = v.value

    # DEA 多空信号
    dea_base = sum([abs(dea[-2] - dea[-1]), abs(dea[-3] - dea[-2]), abs(dea[-4] - dea[-3])]) / 3
    if dea[-1] > dea_base:
        v = Signal(k1=k1, k2=k2, k3="DEA多空", v1="多头")
    elif dea[-1] < -dea_base:
        v = Signal(k1=k1, k2=k2, k3="DEA多空", v1="空头")
    else:
        v = Signal(k1=k1, k2=k2, k3="DEA多空", v1="模糊")
    s[v.key] = v.value

    # DEA 方向信号
    if dea[-1] > dea[-2]:
        v = Signal(k1=k1, k2=k2, k3="DEA方向", v1="向上")
    elif dea[-1] < dea[-2]:
        v = Signal(k1=k1, k2=k2, k3="DEA方向", v1="向下")
    else:
        v = Signal(k1=k1, k2=k2, k3="DEA方向", v1="模糊")
    s[v.key] = v.value

    # MACD 多空信号
    if macd[-1] >= 0:
        v = Signal(k1=k1, k2=k2, k3="MACD多空", v1="多头")
    else:
        v = Signal(k1=k1, k2=k2, k3="MACD多空", v1="空头")
    s[v.key] = v.value

    # MACD 方向信号
    if macd[-1] > macd[-2] > macd[-3]:
        v = Signal(k1=k1, k2=k2, k3="MACD方向", v1="向上")
    elif macd[-1] < macd[-2] < macd[-3]:
        v = Signal(k1=k1, k2=k2, k3="MACD方向", v1="向下")
    else:
        v = Signal(k1=k1, k2=k2, k3="MACD方向", v1="模糊")
    s[v.key] = v.value
    return s


def get_s_sma(c: analyze.CZSC, di: int = 1, t_seq=(5, 10, 20, 60)) -> OrderedDict:
    """获取倒数第i根K线的SMA相关信号"""
    freq: Freq = c.freq
    s = OrderedDict()

    k1 = str(freq.value)
    k2 = f"倒{di}K"
    for t in t_seq:
        x1 = Signal(k1=k1, k2=k2, k3=f"SMA{t}多空", v1="其他", v2='其他', v3='其他')
        x2 = Signal(k1=k1, k2=k2, k3=f"SMA{t}方向", v1="其他", v2='其他', v3='其他')
        s[x1.key] = x1.value
        s[x2.key] = x2.value

    if len(c.bars_raw) < 100:
        return s

    if di == 1:
        close = np.array([x.close for x in c.bars_raw[-100:]])
    else:
        close = np.array([x.close for x in c.bars_raw[-100-di+1:-di+1]])

    for t in t_seq:
        sma = SMA(close, timeperiod=t)
        if close[-1] >= sma[-1]:
            v1 = Signal(k1=k1, k2=k2, k3=f"SMA{t}多空", v1="多头")
        else:
            v1 = Signal(k1=k1, k2=k2, k3=f"SMA{t}多空", v1="空头")
        s[v1.key] = v1.value

        if sma[-1] >= sma[-2]:
            v2 = Signal(k1=k1, k2=k2, k3=f"SMA{t}方向", v1="向上")
        else:
            v2 = Signal(k1=k1, k2=k2, k3=f"SMA{t}方向", v1="向下")
        s[v2.key] = v2.value
    return s





# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/11/7 17:09
describe: 支持分批买入卖出的高级交易员
"""
import os
import webbrowser
from typing import Callable, List
from collections import OrderedDict
from pyecharts.charts import Tab
from pyecharts.components import Table
from pyecharts.options import ComponentTitleOpts

from ..analyze import CZSC
from ..objects import PositionLong, PositionShort, Operate, Signal, Event, RawBar
from ..utils.bar_generator import BarGenerator
from ..utils.cache import home_path


class CzscAdvancedTrader:
    """缠中说禅技术分析理论之多级别联立交易决策类（支持分批开平仓 / 支持从任意周期开始交易）"""

    def __init__(self,
                 bg: BarGenerator,
                 get_signals: Callable,
                 long_events: List[Event] = None,
                 long_pos: PositionLong = None,
                 short_events: List[Event] = None,
                 short_pos: PositionShort = None,
                 max_bi_count: int = 50,
                 bi_min_len: int = 7,
                 signals_n: int = 0,
                 verbose: bool = False,
                 ):
        """

        :param bg: K线合成器
        :param get_signals: 自定义的单级别信号计算函数
        :param long_events: 自定义的多头交易事件组合，推荐平仓事件放到前面
        :param long_pos: 多头仓位对象
        :param short_events: 自定义的空头交易事件组合，推荐平仓事件放到前面
        :param short_pos: 空头仓位对象
        :param max_bi_count: 单个级别最大保存笔的数量
        :param bi_min_len: 一笔最小无包含K线数量
        :param signals_n: 见 `CZSC` 对象
        :param verbose: 是否显示更多信息，默认为False
        """
        self.name = "CzscAdvancedTrader"
        self.bg = bg
        self.base_freq = bg.base_freq
        self.freqs = list(bg.bars.keys())
        self.long_events = long_events
        self.long_pos = long_pos
        self.short_events = short_events
        self.short_pos = short_pos
        self.verbose = verbose
        self.kas = {freq: CZSC(b, max_bi_count=max_bi_count,
                               get_signals=get_signals, signals_n=signals_n,
                               bi_min_len=bi_min_len, verbose=verbose)
                    for freq, b in bg.bars.items()}
        self.s = self._cal_signals()

    def __repr__(self):
        return "<{} for {}>".format(self.name, self.symbol)

    def take_snapshot(self, file_html=None, width: str = "1400px", height: str = "580px"):
        """获取快照

        :param file_html: 交易快照保存的 html 文件名
        :param width: 图表宽度
        :param height: 图表高度
        :return:
        """
        tab = Tab(page_title="{}@{}".format(self.symbol, self.end_dt.strftime("%Y-%m-%d %H:%M")))
        for freq in self.freqs:
            chart = self.kas[freq].to_echarts(width, height)
            tab.add(chart, freq)

        for freq in self.freqs:
            t1 = Table()
            t1.add(["名称", "数据"], [[k, v] for k, v in self.s.items() if k.startswith("{}_".format(freq))])
            t1.set_global_opts(title_opts=ComponentTitleOpts(title="缠中说禅信号表", subtitle=""))
            tab.add(t1, "{}信号表".format(freq))

        t2 = Table()
        ths_ = [["同花顺F10",  "http://basic.10jqka.com.cn/{}".format(self.symbol[:6])]]
        t2.add(["名称", "数据"], [[k, v] for k, v in self.s.items() if "_" not in k] + ths_)
        t2.set_global_opts(title_opts=ComponentTitleOpts(title="缠中说禅因子表", subtitle=""))
        tab.add(t2, "因子表")

        if file_html:
            tab.render(file_html)
        else:
            return tab

    def open_in_browser(self, width="1400px", height="580px"):
        """直接在浏览器中打开分析结果"""
        file_html = os.path.join(home_path, "temp_czsc_advanced_trader.html")
        self.take_snapshot(file_html, width, height)
        webbrowser.open(file_html)

    def get_s_position(self, pos: [PositionLong, PositionShort]):
        """计算多头持仓信号

        :return:
        """
        if isinstance(pos, PositionLong):
            k1 = "多头"
        elif isinstance(pos, PositionShort):
            k1 = "空头"
        else:
            raise ValueError

        s = OrderedDict()
        default_signals = [
            Signal(k1=k1, k2="最大", k3='盈利', v1="其他", v2="其他", v3="其他"),
            Signal(k1=k1, k2="最大", k3='回撤', v1="其他", v2="其他", v3="其他"),
            Signal(k1=k1, k2="最大", k3='回撤盈利比', v1="其他", v2="其他", v3="其他"),

            Signal(k1=k1, k2="累计", k3='盈亏', v1="其他", v2="其他", v3="其他"),
            Signal(k1=k1, k2="持仓", k3='时间', v1="其他", v2="其他", v3="其他"),
            Signal(k1=k1, k2="持仓", k3='基础K线数量', v1="其他", v2="其他", v3="其他"),
        ]
        for signal_ in default_signals:
            s[signal_.key] = signal_.value

        if pos.pos == 0:
            return s

        base_freq = self.base_freq
        latest_price = self.latest_price
        bid = self.bg.bars[base_freq][-1].id
        end_dt = self.bg.bars[base_freq][-1].dt

        if isinstance(pos, PositionLong):
            last_o = [x for x in pos.operates[-50:] if x['op'] == Operate.LO][-1]
            last_o_price = last_o['price']
            yl = pos.long_high / last_o_price - 1                   # 最大盈利
            hc = abs(latest_price / pos.long_high - 1)              # 最大回撤
            yk = (latest_price - last_o_price) / last_o_price       # 累计盈亏
        else:
            last_o = [x for x in pos.operates[-50:] if x['op'] == Operate.SO][-1]
            last_o_price = last_o['price']
            yl = last_o_price / pos.short_low - 1                   # 最大盈利
            hc = abs(pos.short_low / latest_price - 1)              # 最大回撤
            yk = (last_o_price - latest_price) / last_o_price       # 累计盈亏

        last_o_dt = last_o['dt']
        last_o_bid = last_o['bid']

        hc_yl_rate = hc / (yl + 0.000001)                   # 最大回撤盈利比
        hold_time = (end_dt - last_o_dt).total_seconds()   # 持仓时间，单位：秒
        hold_nbar = bid - last_o_bid                       # 持仓基础K线数量
        assert yl >= 0 and hc >= 0 and hc_yl_rate >= 0

        # ----------------------------------------------------------------------------------
        if yl > 0.15:
            v1 = "超过1500BP"
        elif yl > 0.1:
            v1 = "超过1000BP"
        elif yl > 0.08:
            v1 = "超过800BP"
        elif yl > 0.05:
            v1 = "超过500BP"
        elif yl > 0.03:
            v1 = "超过300BP"
        else:
            v1 = "低于300BP"
        v = Signal(k1=k1, k2="最大", k3='盈利', v1=v1)
        s[v.key] = v.value

        # ----------------------------------------------------------------------------------
        if hc > 0.15:
            v1 = "超过1500BP"
        elif hc > 0.1:
            v1 = "超过1000BP"
        elif hc > 0.08:
            v1 = "超过800BP"
        elif hc > 0.05:
            v1 = "超过500BP"
        elif hc > 0.03:
            v1 = "超过300BP"
        else:
            v1 = "低于300BP"
        v = Signal(k1=k1, k2="最大", k3='回撤', v1=v1)
        s[v.key] = v.value

        # ----------------------------------------------------------------------------------
        if hc_yl_rate > 0.8:
            v1 = "大于08"
        elif hc_yl_rate > 0.6:
            v1 = "大于06"
        elif hc_yl_rate > 0.5:
            v1 = "大于05"
        elif hc_yl_rate > 0.3:
            v1 = "大于03"
        else:
            v1 = "小于03"
        v = Signal(k1=k1, k2="最大", k3='回撤盈利比', v1=v1)
        s[v.key] = v.value

        # ----------------------------------------------------------------------------------
        if yk >= 0:
            v1 = "盈利"
        else:
            v1 = "亏损"

        if abs(yk) > 0.15:
            v2 = "超过1500BP"
        elif abs(yk) > 0.1:
            v2 = "超过1000BP"
        elif abs(yk) > 0.08:
            v2 = "超过800BP"
        elif abs(yk) > 0.05:
            v2 = "超过500BP"
        elif abs(yk) > 0.03:
            v2 = "超过300BP"
        else:
            v2 = "低于300BP"
        v = Signal(k1=k1, k2="累计", k3='盈亏', v1=v1, v2=v2)
        s[v.key] = v.value

        # ----------------------------------------------------------------------------------
        if hold_time > 3600 * 24 * 13:
            v1 = "超过13天"
        elif hold_time > 3600 * 24 * 8:
            v1 = "超过8天"
        elif hold_time > 3600 * 24 * 5:
            v1 = "超过5天"
        elif hold_time > 3600 * 24 * 3:
            v1 = "超过3天"
        else:
            v1 = "低于3天"
        v = Signal(k1=k1, k2="持仓", k3='时间', v1=v1)
        s[v.key] = v.value

        # ----------------------------------------------------------------------------------
        if hold_nbar > 300:
            v1 = "超过300根"
        elif hold_nbar > 200:
            v1 = "超过200根"
        elif hold_nbar > 150:
            v1 = "超过150根"
        elif hold_nbar > 100:
            v1 = "超过100根"
        elif hold_nbar > 50:
            v1 = "超过50根"
        else:
            v1 = "低于50根"
        v = Signal(k1=k1, k2="持仓", k3='基础K线数量', v1=v1)
        s[v.key] = v.value

        return s

    def _cal_signals(self):
        """计算信号"""
        base_freq = self.base_freq
        self.symbol = self.kas[base_freq].symbol
        self.end_dt = self.kas[base_freq].bars_raw[-1].dt
        self.bid = self.kas[base_freq].bars_raw[-1].id
        self.latest_price = self.kas[base_freq].bars_raw[-1].close

        s = OrderedDict()
        for freq, ks in self.kas.items():
            s.update(ks.signals)

        s.update(self.kas[base_freq].bars_raw[-1].__dict__)
        if self.long_pos:
            s.update(self.get_s_position(self.long_pos))
        if self.short_pos:
            s.update(self.get_s_position(self.short_pos))
        return s

    def update(self, bar: RawBar):
        """输入基础周期已完成K线，更新信号，更新仓位"""
        self.bg.update(bar)
        for freq, b in self.bg.bars.items():
            self.kas[freq].update(b[-1])
        self.s = self._cal_signals()
        dt = self.end_dt
        price = self.latest_price
        bid = self.bid

        # 遍历 long_events，更新 long_pos
        if self.long_events:
            assert isinstance(self.long_pos, PositionLong), "long_events 必须配合 PositionLong 使用"

            op = Operate.HO
            op_desc = ""

            for event in self.long_events:
                m, f = event.is_match(self.s)
                if m:
                    op = event.operate
                    op_desc = f"{event.name}@{f}"
                    break

            self.long_pos.update(dt, op, price, bid, op_desc)

        # 遍历 short_events，更新 short_pos
        if self.short_events:
            assert isinstance(self.short_pos, PositionShort), "short_events 必须配合 PositionShort 使用"

            op = Operate.HO
            op_desc = ""

            for event in self.short_events:
                m, f = event.is_match(self.s)
                if m:
                    op = event.operate
                    op_desc = f"{event.name}@{f}"
                    break

            self.short_pos.update(dt, op, price, bid, op_desc)





# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2021/8/25 17:43
describe: 成交量相关信号
"""
import numpy as np
from collections import OrderedDict
from czsc import CZSC, Freq, Signal
from czsc.utils.ta import SMA


def get_s_vol_single_sma(c: CZSC, di: int = 1, t_seq=(5, 10, 20, 60)) -> OrderedDict:
    """获取倒数第i根K线的成交量单均线信号"""
    freq: Freq = c.freq
    s = OrderedDict()

    k1 = str(freq.value)
    k2 = f"倒{di}K成交量"
    for t in t_seq:
        x1 = Signal(k1=k1, k2=k2, k3=f"SMA{t}多空", v1="其他", v2='其他', v3='其他')
        x2 = Signal(k1=k1, k2=k2, k3=f"SMA{t}方向", v1="其他", v2='其他', v3='其他')
        s[x1.key] = x1.value
        s[x2.key] = x2.value

    min_k_nums = max(t_seq) + 10
    if len(c.bars_raw) < min_k_nums:
        return s

    if di == 1:
        vol = np.array([x.vol for x in c.bars_raw[-min_k_nums:]], dtype=np.float)
    else:
        vol = np.array([x.vol for x in c.bars_raw[-min_k_nums-di+1:-di+1]], dtype=np.float)

    for t in t_seq:
        sma = SMA(vol[-t-10:], timeperiod=t)
        if vol[-1] >= sma[-1]:
            v1 = Signal(k1=k1, k2=k2, k3=f"SMA{t}多空", v1="多头")
        else:
            v1 = Signal(k1=k1, k2=k2, k3=f"SMA{t}多空", v1="空头")
        s[v1.key] = v1.value

        if sma[-1] >= sma[-2]:
            v2 = Signal(k1=k1, k2=k2, k3=f"SMA{t}方向", v1="向上")
        else:
            v2 = Signal(k1=k1, k2=k2, k3=f"SMA{t}方向", v1="向下")
        s[v2.key] = v2.value
    return s


def get_s_vol_double_sma(c: CZSC, di: int = 1, t1: int = 5, t2: int = 20) -> OrderedDict:
    """获取倒数第i根K线的成交量双均线信号"""
    assert t2 > t1, "t2必须是长线均线，t1为短线均线"
    freq: Freq = c.freq
    s = OrderedDict()

    k1 = str(freq.value)
    k2 = f"倒{di}K成交量"
    k3 = f"{t1}#{t2}双均线"
    v = Signal(k1=k1, k2=k2, k3=k3, v1="其他", v2='其他', v3='其他')
    s[v.key] = v.value

    min_len_bars = t2 + 10
    if len(c.bars_raw) < min_len_bars:
        return s

    if di == 1:
        vol = np.array([x.vol for x in c.bars_raw[-min_len_bars:]], dtype=np.float)
    else:
        vol = np.array([x.vol for x in c.bars_raw[-min_len_bars-di+1: -di+1]], dtype=np.float)

    sma1 = SMA(vol, timeperiod=t1)
    sma2 = SMA(vol, timeperiod=t2)

    if sma2[-1] >= sma1[-1]:
        v = Signal(k1=k1, k2=k2, k3=k3, v1="多头")
    else:
        v = Signal(k1=k1, k2=k2, k3=k3, v1="空头")
    s[v.key] = v.value
    return s


def get_s_amount_n(c: CZSC, di=1, n=10, total_amount=10):
    """N日总成交额信号"""
    s = OrderedDict()
    if c.freq != Freq.D or len(c.bars_raw) <= di + n + 5:
        return s

    k1 = str(c.freq.value)
    k2 = f"倒{di}K成交额"
    k3 = f"近{n}日累计超{total_amount}亿"

    if di == 1:
        bars = c.bars_raw[-n:]
    else:
        bars = c.bars_raw[-n-di+1: -di+1]

    assert len(bars) == n

    n_total_amount = sum([x.amount for x in bars])
    if n_total_amount > (total_amount * 1e8):
        v1 = "是"
    else:
        v1 = "否"

    v = Signal(k1=k1, k2=k2, k3=k3, v1=v1)
    s[v.key] = v.value
    return s



# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2019/10/29 15:01
"""
from . import ai
from . import utils
from . import traders
from . import sensors
from . import aphorism

from .analyze import CZSC
from .traders.advanced import CzscAdvancedTrader
from .utils.ta import SMA, EMA, MACD, KDJ
from .objects import Freq, Operate, Direction, Signal, Factor, Event, RawBar, NewBar

__version__ = "0.8.19"
__author__ = "zengbin93"
__email__ = "zeng_bin8888@163.com"
__date__ = "20220319"

print(f"欢迎使用CZSC！当前版本标识为 {__version__}@{__date__}\n")

aphorism.print_one()




# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2022/1/29 15:01
describe: 相关系数计算、可视化

References:
1. https://zhuanlan.zhihu.com/p/362258222
2. https://blog.csdn.net/qq_45538220/article/details/107429201
"""

import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
from sklearn import metrics
from tqdm import tqdm

plt.rcParams['font.sans-serif'] = ['SimHei']    # 用来正常显示中文标签
plt.rcParams['axes.unicode_minus'] = False      # 用来正常显示负号


def nmi_matrix(df: pd.DataFrame, heatmap=False) -> pd.DataFrame:
    """计算高维标准化互信息并以矩阵形式输出

    :param df: 数据
    :param heatmap: 是否绘制热力图
    :return:
    """
    cols = df.columns.to_list()

    m_dict = {}
    for i, col1 in tqdm(enumerate(cols), desc='nmi'):
        X = df[col1]
        for col2 in cols[i:]:
            Y = df[col2]
            nmi = metrics.normalized_mutual_info_score(X, Y)
            m_dict[f"{col1}_{col2}"] = nmi
            m_dict[f"{col2}_{col1}"] = nmi

    m = []
    for col1 in cols:
        A = []
        for col2 in cols:
            A.append(m_dict[f"{col1}_{col2}"])
        m.append(A)

    dfm = pd.DataFrame(m, index=cols, columns=cols)

    if heatmap:
        print('NMI(标准化互信息) = \n', dfm)
        plt.close()
        figure, ax = plt.subplots(figsize=(len(cols), len(cols)))
        sns.heatmap(dfm, square=True, annot=True, ax=ax)
        plt.show()
    return dfm









# -*- coding: utf-8 -*-
"""
author: zengbin93
email: zeng_bin8888@163.com
create_dt: 2022/2/24 14:17
describe: TsBacktest 单元测试
"""
import sys
sys.path.insert(0, '.')
sys.path.insert(0, '..')

import pandas as pd
from czsc.data.ts_cache import TsDataCache
from czsc.traders.ts_backtest import TsStocksBacktest, TraderPerformance
from examples import tactics

df = pd.read_excel(r"C:\ts_data\trader_stocks_v2_a_t0\trader_stocks_v2_a_t0_check_long_pairs.xlsx")


tp = TraderPerformance(df)
